package net.knarcraft.blacksmith.command;

import net.knarcraft.blacksmith.BlacksmithPlugin;
import net.knarcraft.blacksmith.config.SmithPreset;
import net.knarcraft.blacksmith.config.SmithPresetFilter;
import net.knarcraft.blacksmith.formatting.BlacksmithTranslatableMessage;
import net.knarcraft.knarlib.formatting.StringFormatter;
import org.bukkit.Material;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

/**
 * The command for displaying which materials are contained in a preset
 */
public class PresetCommand implements CommandExecutor {

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label,
                             @NotNull String[] args) {
        if (args.length < 1) {
            return false;
        }

        String presetName = args[0].toUpperCase().replace('-', '_');
        List<Material> includedMaterials;

        try {
            //Display the preset with the filter applied
            if (presetName.contains(":")) {
                String[] parts = presetName.split(":");
                SmithPreset smithPreset = SmithPreset.valueOf(parts[0]);
                SmithPresetFilter filter = SmithPresetFilter.valueOf(parts[1]);

                if (!smithPreset.supportsFilter(filter)) {
                    BlacksmithPlugin.getStringFormatter().displayErrorMessage(sender,
                            BlacksmithTranslatableMessage.INVALID_FILTER_FOR_PRESET);
                    return false;
                }
                includedMaterials = smithPreset.getFilteredMaterials(filter);
            } else {
                includedMaterials = SmithPreset.valueOf(presetName).getMaterials();
            }
        } catch (IllegalArgumentException exception) {
            BlacksmithPlugin.getStringFormatter().displayErrorMessage(sender,
                    BlacksmithTranslatableMessage.INVALID_PRESET_OR_FILTER);
            return false;
        }

        //Convert materials to strings before output
        List<String> materialNames = new ArrayList<>();
        for (Material material : includedMaterials) {
            materialNames.add(material.name());
        }
        BlacksmithPlugin.getStringFormatter().displaySuccessMessage(sender, StringFormatter.replacePlaceholder(
                BlacksmithPlugin.getTranslator().getTranslatedMessage(BlacksmithTranslatableMessage.PRESET_MATERIALS),
                "{materials}", String.join(", ", materialNames)));
        return true;
    }

}
